from typing import List, Callable

import numpy as np

from intervallum.interval import IntervalNumber
from intervallum.box import Box, BoxVector


Func = Callable[[IntervalNumber, BoxVector], BoxVector]


class SatelliteStabilization:
    
    def __init__(self, u1: Func, u2: Func, u3: Func):
        self.u1 = u1
        self.u2 = u2
        self.u3 = u3
    
    def _model_eq(self, t: IntervalNumber, state: BoxVector) -> List[IntervalNumber]:
        [p, q, r, _] = state
        u1 = self.u1(t, state)
        u2 = self.u2(t, state)
        u3 = self.u3(t, state)
        return [
            u1 / 6.0,
            u2 - 0.2 * p * r,
            0.2 * (u3 + p * q),
            abs(u1) + abs(u2) + abs(u3)
        ]
    
    def model_eq_np(self, t: IntervalNumber, state: BoxVector) -> BoxVector:
        return np.array(self._model_eq(t, state))
    
    # not used not
    # def model_eq_interval(self, t: IntervalNumber, state: BoxVector) -> BoxVector:
    #     return Box(*self._model_eq(t, state))



from typing import Union, List

import numpy as np


Points = Union[List, np.ndarray]


class PiecewiseConstantController:

    def __init__(self,
                 time_grid: Points,
                 values: Points,
                 default_value: float = None):
        self._time_grid = list(time_grid)
        self._values = values
        self._default_value = default_value

    def generate(self, t: float) -> float:
        for i, t_prev in enumerate(self._time_grid[:-1]):
            if t_prev <= t < self._time_grid[i + 1]:
                return self._values[i]
        if self._default_value is None:
            raise ValueError(f"Can not generate control for tau <{t}>")
        else:
            return self._default_value

    def __call__(self, *args, **kwargs):
        return self.generate(kwargs.get("t", args[0]))



import os
from setuptools import setup, find_packages

if os.path.exists("requirements.txt"):
    with open("requirements.txt") as f:
        required = f.read().splitlines()
else:
    required = []

setup(
    name="cybernetics",
    version="0.1.2",

    description="Cybernetics - package with various dynamic models and cybernetics tools",

    url="https://github.com/wol4aravio/Cybernetics",

    author="Panovskiy Valentin",
    author_email="panovskiy.v@yandex.ru",

    license="MIT License",

    classifiers=[
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Mathematics",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
    ],

    keywords="",

    packages=find_packages(exclude=["tests"]),

    install_requires=required

)



from typing import Union, List

import numpy as np


Points = Union[List, np.ndarray]


class PiecewiseLinearController:

    def __init__(self,
                 time_grid: Points,
                 values: Points,
                 default_value: float = None):
        self._time_grid = time_grid
        self._values = values
        self._default_value = default_value

    def generate(self, t: float) -> float:
        for i, t0 in enumerate(self._time_grid[:-1]):
            t1 = self._time_grid[i + 1]
            if t0 <= t <= t1:
                u0, u1 = self._values[i], self._values[i + 1]
                return ((t1 - t) * u0 + (t - t0) * u1) / (t1 - t0)
        if self._default_value is None:
            raise ValueError(f"Can not generate control for tau <{t}>")
        else:
            return self._default_value

    def __call__(self, *args, **kwargs):
        return self.generate(kwargs.get("t", args[0]))
